#!/usr/bin/perl
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use lib "t/lib";
use MooseCompat;

use strict;
use warnings;

use Test::More;

use Test::Requires {
    'Test::Output' => '0.01', # skip all if not installed
};

{
    package Foo;
    use Mouse;
    __PACKAGE__->meta->make_immutable;
}

{
    package Bar;
    use Mouse;

    extends 'Foo';

    __PACKAGE__->meta->make_immutable;
    __PACKAGE__->meta->make_mutable;


    # This actually is testing for a bug in Mouse::Meta that cause
    # Mouse::Meta::Method to spit out a warning when it
    # shouldn't have done so. The bug was fixed in CMOP 0.75.
    ::stderr_unlike(
        sub { Bar->meta->make_immutable },
        qr/Not inlining a constructor/,
        'no warning that Bar may not have an inlined constructor'
    );
}

done_testing;
