use strict;
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use lib "t/lib";
use MooseCompat;
use warnings;

use Test::Exception;
use Test::More;

{

    package FakeBar;
    use Mouse::Role;

    around isa => sub {
        my ( $orig, $self, $v ) = @_;
        return 1 if $v eq 'Bar';
        return $orig->( $self, $v );
    };

    package Foo;
    use Mouse;

    use Test::More; # for $TODO

    ::lives_ok { with 'FakeBar' } 'applied role';

    my $foo = Foo->new;
    ::isa_ok $foo, 'Bar';
}

done_testing;
