#!/usr/bin/env perl
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use lib "t/lib";
use MooseCompat;
use strict;
use warnings;
use Test::More;

{
    package Foo::Base::Meta::Trait;
    use Mouse::Role;
}

{
    package Foo::Base;
    use Mouse;
    Mouse::Util::MetaRole::apply_metaroles(
        for             => __PACKAGE__,
        class_metaroles => { constructor => ['Foo::Base::Meta::Trait'] },
    );
    __PACKAGE__->meta->make_immutable;
}

{
    package Foo::Meta::Trait;
    use Mouse::Role;
}

{
    package Foo;
    use Mouse;
    Mouse::Util::MetaRole::apply_metaroles(
        for             => __PACKAGE__,
        class_metaroles => { constructor => ['Foo::Meta::Trait'] }
    );
    ::ok(!Foo->meta->is_immutable);
    extends 'Foo::Base';
    ::ok(!Foo->meta->is_immutable);
}

done_testing;
